<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Twig\Extension;

use Dedi\SyliusImageOptimizerPlugin\Resolver\ImageResolverInterface;
use Twig\Environment;
use Twig\TwigFunction;
use Twig_Extension;

class RenderPictureExtension extends Twig_Extension
{
    private readonly Environment $engine;

    private readonly ImageResolverInterface $imageResolver;

    private readonly string $defaultCustomizeTemplate;

    private readonly string $defaultTemplate;

    public function __construct(
        Environment $engine,
        ImageResolverInterface $imageResolver,
        string $defaultTemplate,
        string $defaultCustomizeTemplate
    ) {
        $this->engine = $engine;
        $this->imageResolver = $imageResolver;
        $this->defaultTemplate = $defaultTemplate;
        $this->defaultCustomizeTemplate = $defaultCustomizeTemplate;
    }

    public function getFunctions()
    {
        return [
            new TwigFunction('dedi_sylius_image_optimizer_for_image', [$this, 'renderImageOptimizerForImage'], [
                'is_safe' => ['html'],
            ]),
            new TwigFunction('dedi_sylius_image_optimizer_resolve_image_path', [$this->imageResolver, 'resolve']),
        ];
    }

    public function renderImageOptimizerForImage(
        string $image,
        string $alternativeText = '',
        array $attributes = null,
        array $runtimeConfigurationImage = null,
        ?string $template = null
    ): string {
        if ($runtimeConfigurationImage === null) {
            return $this->engine->render($template ?? $this->defaultTemplate, [
                'image' => $image,
                'alternativeText' => $alternativeText,
                'attrs' => $attributes,
            ]);
        }

        return $this->engine->render($template ?? $this->defaultCustomizeTemplate, [
            'image' => $image,
            'alternativeText' => $alternativeText,
            'attrs' => $attributes,
            'runtimeConfigurationImage' => $runtimeConfigurationImage,
        ]);
    }
}
